function test_ip_plot(d,m,isAcute,isJL,isNegate)
%
% Empricial test of inner product preservation in Figure 2 (b)(d)
%
% Usage:
%       test_ip_plot(d,m,isAcute,isJL)
%
% Input:
%    d:         the orignial dimesnion of the input vector. d = 300 in the
%               paper.
%    m:         the number of random matrices to be generated. m = 2000 in
%               the paper. To speed up, one can reduce m.
%    isAcute:   testing acute angle when 1, abtuse angle when 0.
%    isJL:      testing JL lemma case or not
%    isNegate:  testing the case of <w,-x> instead of <w,x>


%close all

if nargin<3
    fprintf('\nNot enough input, thus use the default setting---------------\n');
    d = 300;
    m = 100;
    isAcute = 0;
    isJL = 0;
    isNegate= 0;
end

if nargin<4
    isJL = 0;
end
if nargin < 5
    isNegate = 0;
end

figure;
set(gca,'FontSize',15)
x_idx = (1:10);
x = 0.1*d*x_idx;
size_x = numel(x);
set(gca,'XTick',(((x_idx))'));
set(gca,'XTickLabel',num2str(x(x_idx)'));
hold on




colors = {'-ro','--ms',':b>','-.c*','-.gd','-kp'}


cnt = 0;

%RandStream.setDefaultStream ...
%    (RandStream('mt19937ar','seed',1)); %used in the paper. Now use RandStream.setGlobalStream to support matlab 2013b

RandStream.setGlobalStream(RandStream('mt19937ar','seed',1));

gamma_all = {};
for gamma = [0.8,0.4]
    
    
    w = randn(d,1);
    if(isJL)
        x = w;
        ip = w'*x;
    else
        if(isAcute)
            x = w+3*(1-gamma)*randn(d,1);
            angle = w'*x/(norm(w,2)*norm(x,2));
            while(angle < gamma)
                x = w+3*(1-gamma)*randn(d,1);
                angle = w'*x/(norm(w,2)*norm(x,2));
            end
        else
            x = randn(d,1);
            angle = w'*x/(norm(w,2)*norm(x,2));
        end
    end
    
    if (isNegate)
        x = -x; % to test obstuse case close to zero.
    end
    angle = w'*x/(norm(w,2)*norm(x,2));
    gamma_all{end+1} = sprintf('%6.3g',angle);
    
    %for epsi = 0.1:0.2:0.5
    for epsi = [0.3,0.1]
        cnt = cnt + 1;
        
        prac_pro2_all = [];
        n_all = [];
        for j = 0.1:0.1:1
            Q = [];
            Q_ip = [];
            
            n = floor(j*d);
            
            for i = 1:m
                R = randn(n,d)/sqrt(n);
                rw = R*w;
                rx = R*x;
                Q_ip(end+1) = rw'*rx;
                
            end
            mean_Q_ip = mean(Q_ip);
            var_Q_ip = var(Q_ip);
            
            rate = Q_ip/(w'*x);
            
            cnt2 = numel((find(( rate<(1-epsi))|(rate > (1+epsi)))));
            prac_pro2 = cnt2/m;
            prac_pro2_all(end+1) = prac_pro2;
            n_all(end+1) = n;
            
            fprintf('\n------------ <w,x> = %g, n = %g(/%g), epsi = %g, angle = %g >= gamma (%g)---------------\n',w'*x,n,d,epsi,angle,gamma);
            fprintf('\t Pr( (<Rw,Rx> < <w,x>*%g) | (<Rw,Rx> > <w,x>*%g) )_emp = %g, JL_bound <= %g \n',1-epsi,1+epsi, prac_pro2, exp(-n/4*epsi^2))
            
            
        end
        plot(prac_pro2_all,colors{cnt},'LineWidth',2,'MarkerSize',10);
        hold on;
        
    end
    
end

ylabel('P')
xlabel('n')

legend(['\gamma = ',gamma_all{1}, ', \epsilon = 0.3'],['\gamma = ',gamma_all{1}, ', \epsilon = 0.1'],['\gamma = ',gamma_all{2}, ', \epsilon = 0.3'],['\gamma = ',gamma_all{2}, ', \epsilon = 0.1'],1)

end